package net.sergeych.kiloparsec.adapters

import com.ionspin.kotlin.crypto.util.decodeFromUByteArray
import com.ionspin.kotlin.crypto.util.encodeToUByteArray
import kotlinx.coroutines.cancel
import kotlinx.coroutines.coroutineScope
import kotlinx.coroutines.flow.cancellable
import kotlinx.coroutines.launch
import kotlinx.coroutines.test.runTest
import kotlinx.coroutines.yield
import net.sergeych.kiloparsec.adapter.UdpServer
import net.sergeych.kiloparsec.adapter.acceptTcpDevice
import net.sergeych.kiloparsec.adapter.connectTcpDevice
import net.sergeych.kiloparsec.adapter.toNetworkAddress
import net.sergeych.mp_logger.Log
import org.junit.jupiter.api.Assertions.assertEquals
import kotlin.test.Test

class NetworkTest {

    @Test
    fun udpProviderTest() = runTest {
        Log.connectConsole(Log.Level.DEBUG)
        val s1 = UdpServer(17120)
        val s2 = UdpServer(17121)
        s1.send("Hello".encodeToUByteArray(), "localhost", 17121)
        val d1 = s2.incoming.receive()
        assertEquals(d1.address.port, 17120)
        assertEquals("Hello", d1.message.toByteArray().decodeToString())
        s1.send("world".encodeToUByteArray(), d1.address)
        assertEquals("world", s1.incoming.receive().message.toByteArray().decodeToString())
        //        println("s1: ${s1.bindAddress()}")

    }

    @Test
    fun tcpAsyncConnectionTest() = runTest {
        Log.connectConsole(Log.Level.DEBUG)

        coroutineScope {
            val serverFlow = acceptTcpDevice(17171).cancellable()
            launch {
                serverFlow.collect { device ->
                    println("collected input: $device")
                    device.output.send("Hello, world!".encodeToUByteArray())
                    device.output.send("Great".encodeToUByteArray())
                    while(true) {
                        val x = device.input.receive()?.decodeFromUByteArray() ?: break
                        println("!****************** $x")
                        if( x== "Goodbye" ) break
                        if( x == "die") {
                            println("request death")
                            cancel()
                            break
                        }
                        println("ignoring unexpected input: $x")
                    }
                }
            }
            yield()
            var s = connectTcpDevice("127.0.1.1:17171".toNetworkAddress())
            assertEquals("Hello, world!", s.input.receive()!!.decodeFromUByteArray())
            assertEquals("Great", s.input.receive()!!.decodeFromUByteArray())
            s.output.send("Goodbye".encodeToUByteArray())
            s.close()

            println("connecting- -----------------------------------------------------------------")
            s = connectTcpDevice("127.0.1.1:17171".toNetworkAddress())
            assertEquals("Hello, world!", s.input.receive()!!.decodeFromUByteArray())
            println("--1")
            assertEquals("Great", s.input.receive()!!.decodeFromUByteArray())
            println("--2")
            s.output.send("die".encodeToUByteArray())
//            s.close()
        }
    }
}